@@ -4,17 +4,20 @@ package MOP;
 use strict;
 use warnings;
 
-our $VERSION   = '0.09';
+our $VERSION   = '0.10';
 our $AUTHORITY = 'cpan:STEVAN';
 
 use MOP::Role;
 use MOP::Class;
 
 use MOP::Slot;
-use MOP::Method;
-
 use MOP::Slot::Initializer;
 
+use MOP::Method;
+use MOP::Method::Attribute;
+
+use MOP::Util;
+
 1;
 
 __END__
@@ -27,7 +30,7 @@ MOP - A Meta Object Protocol for Perl 5
 
 =head1 VERSION
 
-version 0.09
+version 0.10
 
 =head1 SYNOPSIS
 
